local gameScreen = {}

local interface = require "interface" 
local tween = require "Tween/tween"

local intendedWidth = 1920
local intendedHeight = 1080
local leftWave
local rightWave
local finalWave
local wavesYPos = 150
local diff = 570 - 150
wavesYPos = wavesYPos + diff
local accumulator = 0
local numberOfPoints = 500
local initialX = 127
local finalX = 1665
local step = (finalX) / numberOfPoints
local fontBig
local fontSmall
local angleStep = 0.4
local velocity = {vel = 0.01}

local firstPattern = require "patterns/firstPattern"
local patternsDifficultyLow = require "patterns/patternDifficultyLow"
local patternsDifficultyMid = require "patterns/patternDifficultyMid"
local patternsDifficultyHigh = require "patterns/patternDifficultyHigh"
local currentPatternData
local level

local patternBox = {x = 0.5 * intendedWidth - 200, y = wavesYPos - 100, width = 400, height = 200}
local patternMatched
local timeToRecognizeNewPattern
local musicLayer1
local musicLayer2
local musicLayer3

local patternsAchievedInCurrentLevel
local numPatternsInLevel
local patternImage0
local patternImagesDifficultyLow
local patternImagesDifficultyMid
local patternImagesDifficultyHigh
local infoForNewPattern
local alfaForPatternTween
local scaleForPatternTween
local velocityTween

local patternMatchedRectanglesImg
local patternMatchedRectangleScaleTween
local patternMatchedRectangleAlfaTween
local patternMatchedRectangleInfo

local levelTextInfo
local levelTextInfoTweenY
local levelTextInfoTweenAlfaIn
local levelTextInfoTweenAlfaOut
local level1TextImg
local level2TextImg
local level3TextImg
local finalTextImg

local titleVisible
local alfaTitle
local alfaTitleTween
local titleImg
local deactivateTitle
local scaleFactor
local gameEnd
local endingLock

local brownNoise

local function reset()
	musicLayer2:setVolume(0)
	musicLayer3:setVolume(0)
	level = 0
	numPatternsInLevel = 1
	currentPatternData = firstPattern.getData()
	patternsAchievedInCurrentLevel = 0
	scaleFactor = 1
	gameEnd = false
	endingLock = true
end

function gameScreen.load()
	interface.load()
	titleVisible = true
	deactivateTitle = false
	alfaTitle = {value = 255}
	titleImg = love.graphics.newImage("sprites/title.png")
	fontBig = love.graphics.newFont("fonts/Launica.ttf", 50)
	fontSmall = love.graphics.newFont("fonts/Launica.ttf", 10)
	leftWave = {}
	rightWave = {}
	finalWave = {}
	local pointsCounter = 1
	local xVal = initialX
	for i = 1, numberOfPoints + 1 do
		leftWave[pointsCounter] = xVal
		rightWave[pointsCounter] = xVal
		finalWave[pointsCounter] = xVal
		xVal = xVal + step
		pointsCounter = pointsCounter + 1
		leftWave[pointsCounter] =  wavesYPos
		rightWave[pointsCounter] = wavesYPos
		finalWave[pointsCounter] = leftWave[pointsCounter] + rightWave[pointsCounter] - wavesYPos
		pointsCounter = pointsCounter + 1
	end
	patternMatched = false
	timeToRecognizeNewPattern = 0
	level = 0
	numPatternsInLevel = 1
	-- currentPatternData = patternsDifficultyHigh.getData(6)
	currentPatternData = firstPattern.getData()
	patternsAchievedInCurrentLevel = 0
	-- patternsAchievedInCurrentLevel = 5
	
	musicLayer1 = love.audio.newSource("sounds/Capa1.ogg")
  	musicLayer1:setLooping(true)
  	musicLayer1:play()

  	musicLayer2 = love.audio.newSource("sounds/Capa2.ogg")
  	musicLayer2:setLooping(true)
  	musicLayer2:setVolume(0)

  	musicLayer3 = love.audio.newSource("sounds/Capa3.ogg")
  	musicLayer3:setLooping(true)
  	musicLayer3:setVolume(0)

  	patternImage0 = love.graphics.newImage("sprites/p0.png")
  	patternImagesDifficultyLow = {}
  	for i = 1, 8 do
  		patternImagesDifficultyLow[i] = love.graphics.newImage("sprites/pdl"..i..".png")
  	end
  	 patternImagesDifficultyMid = {}
  	for i = 1, 13 do
  		patternImagesDifficultyMid[i] = love.graphics.newImage("sprites/pdm"..i..".png")
  	end
  	patternImagesDifficultyHigh = {}
  	for i = 1, 6 do
		patternImagesDifficultyHigh[i] = love.graphics.newImage("sprites/pdh"..i..".png")
  	end

  	infoForNewPattern = {alfa = 255, scale = 1}
  	patternMatchedRectanglesImg = love.graphics.newImage("sprites/greenBox.png")
  	patternMatchedRectangleInfo = {x = -2000, y = -2000, alfa = 0, scale = 1}

  	level1TextImg = love.graphics.newImage("sprites/level1Text.png")
  	level2TextImg = love.graphics.newImage("sprites/level2Text.png")
  	level3TextImg = love.graphics.newImage("sprites/level3Text.png")
  	levelTextInfo = {img = level1TextImg, x = 600, y = 800, alfa = 0}

  	finalTextImg = love.graphics.newImage("sprites/finalText.png")
  	gameEnd = false
  	endingLock = true
  	scaleFactor = 1

  	brownNoise = love.audio.newSource("sounds/Interference.wav", "static")
end

function gameScreen.draw(extrapolationFactor)
	love.graphics.setColor(255, 255, 255)
	interface.draw(scaleFactor)

	love.graphics.setColor(0, 255, 0, patternMatchedRectangleInfo.alfa)
	love.graphics.draw(patternMatchedRectanglesImg, patternMatchedRectangleInfo.x, patternMatchedRectangleInfo.y, 0, patternMatchedRectangleInfo.scale, patternMatchedRectangleInfo.scale, 0.5 * patternMatchedRectanglesImg:getWidth(), 0.5 * patternMatchedRectanglesImg:getHeight())

	if not titleVisible then
		love.graphics.setColor(255, 255, 255, infoForNewPattern.alfa)
		if level == 0 then
			love.graphics.draw(patternImage0, 0.5 * intendedWidth, wavesYPos, 0, infoForNewPattern.scale, infoForNewPattern.scale, 0.5 * patternImage0:getWidth(), 0.5 * patternImage0:getHeight())
		elseif level == 1 then
			love.graphics.draw(patternImagesDifficultyLow[patternsAchievedInCurrentLevel + 1], 0.5 * intendedWidth, wavesYPos, 0, infoForNewPattern.scale, infoForNewPattern.scale, 0.5 * patternImagesDifficultyLow[patternsAchievedInCurrentLevel + 1]:getWidth(), 0.5 * patternImagesDifficultyLow[patternsAchievedInCurrentLevel + 1]:getHeight())
		elseif level == 2 then
			love.graphics.draw(patternImagesDifficultyMid[patternsAchievedInCurrentLevel + 1], 0.5 * intendedWidth, wavesYPos, 0, infoForNewPattern.scale, infoForNewPattern.scale, 0.5 * patternImagesDifficultyMid[patternsAchievedInCurrentLevel + 1]:getWidth(), 0.5 * patternImagesDifficultyMid[patternsAchievedInCurrentLevel + 1]:getHeight())
		elseif level == 3 then
			love.graphics.draw(patternImagesDifficultyHigh[patternsAchievedInCurrentLevel + 1], 0.5 * intendedWidth, wavesYPos, 0, infoForNewPattern.scale, infoForNewPattern.scale, 0.5 * patternImagesDifficultyHigh[patternsAchievedInCurrentLevel + 1]:getWidth(), 0.5 * patternImagesDifficultyHigh[patternsAchievedInCurrentLevel + 1]:getHeight())
		end
	
	love.graphics.setColor(118, 45, 137)

	love.graphics.setLineWidth(7)
	love.graphics.line(unpack(finalWave))
	love.graphics.setFont(fontBig)
	-- love.graphics.print("Current FPS: ".. tostring(love.timer.getFPS()), 0, 0)
	end
	love.graphics.setLineWidth(5)
	love.graphics.rectangle("line", 0.5 * intendedWidth - 200, wavesYPos - 100, 400, 200)

	if not titleVisible then
		love.graphics.setColor(255, 255, 255, levelTextInfo.alfa)
		love.graphics.draw(levelTextInfo.img, levelTextInfo.x, levelTextInfo.y)
	end

	love.graphics.setColor(255, 255, 255, alfaTitle.value)
	love.graphics.draw(titleImg, 0.5 * intendedWidth - 0.5 * titleImg:getWidth(), wavesYPos - 0.5 * titleImg:getHeight() - 60)
end

local function updateLeftWave()
	local newPoints = {}
	for k = 1, #leftWave do
		if k % 2 == 1 then
			newPoints[k] = leftWave[k]
		else
			if k > 2 then
				newPoints[k] = leftWave[k - 2]
			end
		end
	end
	newPoints[2] = wavesYPos
	leftWave = newPoints
end

local function updateRightWave()
	local newPoints = {}
	for k = 1, #rightWave-2, 1 do
		if k % 2 == 1 then
			newPoints[k] = rightWave[k]
		else
			newPoints[k] = rightWave[k + 2]
		end
	end
	newPoints[#rightWave] = wavesYPos
	newPoints[#rightWave-1] = rightWave[1]
	rightWave = newPoints
end

local function getNumPatternsInLevel(level)
	local numPatterns = 0
	if level == 1 then
		numPatterns =  8
	elseif level == 2 then
		numPatterns = 13
	elseif level == 3 then
		numPatterns = 5
	end
	return numPatterns
end

-- Change pattern (and level if necessary)
local function advanceGameplay()
	patternMatchedRectangleInfo.x = 0.5 * intendedWidth
	patternMatchedRectangleInfo.y = wavesYPos
	patternMatchedRectangleInfo.scale = 1
	patternMatchedRectangleInfo.alfa = 255
	patternMatchedRectangleScaleTween = tween.new(1, patternMatchedRectangleInfo, {scale = 2}, "inOutCubic")
	patternMatchedRectangleAlfaTween = tween.new(1, patternMatchedRectangleInfo, {alfa = 0}, "inOutCubic")
	velocity.vel = 0.04
	infoForNewPattern.alfa = 0
	infoForNewPattern.scale = 0
	velocityTween = tween.new(2, velocity, {vel = 0.01}, "inOutExpo")
	alfaForPatternTween = tween.new(4, infoForNewPattern, {alfa = 255}, "inOutElastic")
	scaleForPatternTween = tween.new(4, infoForNewPattern, {scale = 1.0}, "inOutElastic")
	patternsAchievedInCurrentLevel = patternsAchievedInCurrentLevel + 1
	if patternsAchievedInCurrentLevel >= numPatternsInLevel then
		patternsAchievedInCurrentLevel = 0
		level = level + 1
		-- print("Level: ", level)
		numPatternsInLevel = getNumPatternsInLevel(level)
		if level == 1 then
			scaleFactor = 1.1
			currentPatternData = patternsDifficultyLow.getData(1)
			levelTextInfo.img = level1TextImg
			levelTextInfo.x = 0.5 * intendedWidth - 0.5 * level1TextImg:getWidth()
			levelTextInfo.y = 700
			levelTextInfo.alfa = 0
			levelTextInfoTweenY = tween.new(6, levelTextInfo, {y = 800}, "outExpo")
			levelTextInfoTweenAlfaIn = tween.new(3, levelTextInfo, {alfa = 255})
			levelTextInfoTweenAlfaOut = tween.new(3, levelTextInfo, {alfa = 0})
		elseif level == 2 then
			scaleFactor = 1.2
			currentPatternData = patternsDifficultyMid.getData(1)
			musicLayer2:setVolume(100)
			levelTextInfo.img = level2TextImg
			levelTextInfo.x = 0.5 * intendedWidth - 0.5 * level2TextImg:getWidth()
			levelTextInfo.y = 700
			levelTextInfo.alfa = 0
			levelTextInfoTweenY = tween.new(6, levelTextInfo, {y = 800}, "outExpo")
			levelTextInfoTweenAlfaIn = tween.new(3, levelTextInfo, {alfa = 255})
			levelTextInfoTweenAlfaOut = tween.new(3, levelTextInfo, {alfa = 0})

		elseif level == 3 then
			scaleFactor = 1.3
			currentPatternData = patternsDifficultyHigh.getData(1)
			musicLayer3:setVolume(100)
			levelTextInfo.img = level3TextImg
			levelTextInfo.x = 0.5 * intendedWidth - 0.5 * level3TextImg:getWidth()
			levelTextInfo.y = 700
			levelTextInfo.alfa = 0
			levelTextInfoTweenY = tween.new(6, levelTextInfo, {y = 800}, "outExpo")
			levelTextInfoTweenAlfaIn = tween.new(3, levelTextInfo, {alfa = 255})
			levelTextInfoTweenAlfaOut = tween.new(3, levelTextInfo, {alfa = 0})
		else
			gameEnd = true
			levelTextInfo.img = finalTextImg
			levelTextInfo.x = 0.5 * intendedWidth - 0.5 * level3TextImg:getWidth()
			levelTextInfo.y = 700
			levelTextInfo.alfa = 0
			levelTextInfoTweenY = tween.new(6, levelTextInfo, {y = 800}, "outExpo")
			levelTextInfoTweenAlfaIn = tween.new(0.5, levelTextInfo, {alfa = 255})
			levelTextInfoTweenAlfaOut = tween.new(0.5, levelTextInfo, {alfa = 0})
		end
	else
		-- print("Next pattern of level ", level, " is ", patternsAchievedInCurrentLevel + 1)
		if level == 1 then
			currentPatternData = patternsDifficultyLow.getData(patternsAchievedInCurrentLevel + 1)
		elseif level == 2 then
			currentPatternData = patternsDifficultyMid.getData(patternsAchievedInCurrentLevel + 1)
		elseif level == 3 then
			currentPatternData = patternsDifficultyHigh.getData(patternsAchievedInCurrentLevel + 1)
		end
	end
end

local function updateWave()
	updateRightWave()
	updateLeftWave()
	local absoluteError = 0
	local numberOfElements = 0
	local iteratorPattern = 2
	for i = 1, #finalWave do
		if i % 2 == 0 then
			finalWave[i] = leftWave[i] + rightWave[i] - wavesYPos
		end
		if i % 2 == 1 then
			if finalWave[i] >= patternBox.x and finalWave[i] <= patternBox.x + patternBox.width then
				numberOfElements = numberOfElements + 1
				local localError = math.abs(finalWave[i + 1] - currentPatternData[iteratorPattern] - diff)
				if localError <= 20 then localError = 0 end
				absoluteError = absoluteError + localError
				iteratorPattern = iteratorPattern + 2
			end
		end
	end
	absoluteError = absoluteError / numberOfElements
	-- print(absoluteError)
	local errorThreshold = 0.02
	if level == 1 then 
		errorThreshold = 0.1
	elseif level == 2 then
		errorThreshold = 2.0
	elseif level == 3 then
		errorThreshold = 4.5
	end

	if absoluteError <= errorThreshold and not patternMatched then
		-- print("YEAHH!!")
		brownNoise:play()
		patternMatched = true
		timeToRecognizeNewPattern = 5
		advanceGameplay()
	end
end

updating = true
function gameScreen.update(dt)
	interface.update(dt)
	if not titleVisible and not gameEnd then
		if alfaForPatternTween then alfaForPatternTween:update(dt) end
		if scaleForPatternTween then scaleForPatternTween:update(dt) end
		if patternMatchedRectangleScaleTween then patternMatchedRectangleScaleTween:update(dt) end
		if patternMatchedRectangleAlfaTween then  patternMatchedRectangleAlfaTween:update(dt) end
		if levelTextInfoTweenY then levelTextInfoTweenY:update(dt) end
		if alfaTitleTween then alfaTitleTween:update(dt) end
		if levelTextInfoTweenAlfaIn then
			local finished = levelTextInfoTweenAlfaIn:update(dt) 
			if finished and levelTextInfoTweenAlfaOut then
				levelTextInfoTweenAlfaOut:update(dt)
			end
		end
		if velocityTween then velocityTween:update(dt) end
		accumulator = accumulator + dt
		if accumulator >= velocity.vel then--0.02 then
			accumulator = 0
			if timeToRecognizeNewPattern <= 4.5 and updating then updateWave() end
		end
		if timeToRecognizeNewPattern > 0 then
			timeToRecognizeNewPattern = timeToRecognizeNewPattern - dt
		else
			timeToRecognizeNewPattern = 0
			patternMatched = false
		end
	-- Title
	elseif not gameEnd then
		if deactivateTitle then
			titleVisible = false
			alfaTitleTween = tween.new(2, alfaTitle, {value = 0}, "linear")
			musicLayer1:play()
			musicLayer2:play()
			musicLayer3:play()
		end		
	-- End
	else
		if endingLock then endingLock = false alfaTitleTween = tween.new(2, alfaTitle, {value = 255}, "linear") end
		if alfaForPatternTween then alfaForPatternTween:update(dt) end
		if scaleForPatternTween then scaleForPatternTween:update(dt) end
		if patternMatchedRectangleScaleTween then patternMatchedRectangleScaleTween:update(dt) end
		if patternMatchedRectangleAlfaTween then  patternMatchedRectangleAlfaTween:update(dt) end
		if levelTextInfoTweenY then levelTextInfoTweenY:update(dt) end
		if alfaTitleTween then
			local completed = alfaTitleTween:update(dt)
			if completed then
				titleVisible = true
				deactivateTitle = false
				reset()
			end
		end
		if levelTextInfoTweenAlfaIn then
			local finished = levelTextInfoTweenAlfaIn:update(dt) 
			if finished and levelTextInfoTweenAlfaOut then
				levelTextInfoTweenAlfaOut:update(dt)
			end
		end
	end
end

local function generateRightWave(amplitude, offsetX)
	local xVal = offsetX
	local angle = 0.5 * math.pi
	-- localize point index from which we have to change
	local pointIndex = 0
	for k = 1, #rightWave, 2 do
		if rightWave[k] >= offsetX then
			-- print(k, rightWave[k], offsetX)
			-- print(k + 2, rightWave[k + 2], offsetX)
			pointIndex = k
			break
		end
	end
	local pointsCounter = pointIndex
	while angle <= (5 * math.pi * 0.5) do
		local yVal = amplitude * math.sin(angle) + wavesYPos - amplitude
		angle = angle + angleStep
		rightWave[pointsCounter] = xVal
		pointsCounter = pointsCounter + 1
		rightWave[pointsCounter] = yVal
		pointsCounter = pointsCounter + 1
		xVal = xVal + step
	end
end

local function generateLeftWave(amplitude, offsetX)
	local xVal = offsetX
	local angle = 0.5 * math.pi
	-- localize point index from which we have to change
	local pointIndex = 0
	for k = 1, #leftWave, 2 do
		if leftWave[k] >= offsetX then
			pointIndex = k
			-- print(k, leftWave[k])
			break
		end
	end
	local pointsCounter = pointIndex
	-- for i = pointIndex, pointIndex + 100000 do
	while angle <= (5 * math.pi * 0.5) do
		-- if angle > (5 * math.pi * 0.5) then break end
		-- Calculate x of i position
		local yVal = amplitude * math.sin(angle) + wavesYPos - amplitude
		angle = angle + angleStep
		leftWave[pointsCounter] = xVal
		pointsCounter = pointsCounter + 1
		leftWave[pointsCounter] = yVal
		pointsCounter = pointsCounter + 1
		xVal = xVal + step
	end
end

patternNumber = 4
local function captureWave()
	local file = love.filesystem.newFile("pattern "..patternNumber..".lua")
	local ok, err = file:open( "a" )
	if not ok then
		print(err)
	else
		local firstWrite = true
		file:write("patternData["..patternNumber.."]".."= {\n")
		for k, point in ipairs(finalWave) do
			if k % 2 == 1 then
				if point >= patternBox.x and point <= patternBox.x + patternBox.width then
					if firstWrite then
						firstWrite = false
						file:write(point..","..finalWave[k + 1].."\n")
					else
						file:write(","..point..","..finalWave[k + 1].."\n")
					end
				end
			end 
		end
		file:write("\n}")
		file:close()
	end
	patternNumber = patternNumber + 1
end

function gameScreen.keypressed(key)
	interface.keypressed(key)
	if titleVisible then
		if key == "escape" then
			love.event.quit()
		else
			deactivateTitle = true
		end
	else
		if key == "escape" then
			titleVisible = true
			deactivateTitle = false
			alfaTitle.value = 255
			reset()
		end
		-- if key == "1" then condition = not condition end
		-- if key == "q" then love.event.quit() end
		if key == "return" then
			generateRightWave(25 * interface.getDirectionRightWave(), finalX + 75)
		end
		if key == "space" then
			generateLeftWave(25 * interface.getDirectionLeftWave(), 0)
		end
		--[[if key == "p" then
			captureWave()
		end
		if key == "u" then
			updateWave()
		end]]
		--[[if key == "space" then
			updating = not updating
		end]]
		--[[if key == "2" then
			musicLayer2:setVolume(100)
		end
		if key == "3" then
			musicLayer3:setVolume(100)
		end]]
	end
end

function gameScreen.keyreleased(key)
	-- currentScreen.keyreleased(key)
end

return gameScreen