local interface = {}

local tween = require "Tween/tween"
local background
local backgroundPulse
local brainGlow1
local brainGlow2
local pulseScale
local pulseScaleUp
local patternBox
local patternImage
local pulse1
local pulse2
local pulse1reverse
local pulse2reverse
local pulse1Up
local pulse2Up
local brainLight
local rightBrainLights
local leftBrainLights
local rectanglesLeft
local rectanglesRight
local backgroundRotationTweenLeft
local backgroundRotationTweenRight
local backgroundRotation

function interface.load()
	background = love.graphics.newImage("sprites/background.png")
	backgroundPulse = love.graphics.newImage("sprites/backgroundPulse.png")
	brainGlow1 = love.graphics.newImage("sprites/brainGlow1.png")
	brainGlow2 = love.graphics.newImage("sprites/brainGlow2.png")
	patternBox = love.graphics.newImage("sprites/patternBox.png")
	patternImage = love.graphics.newImage("sprites/pdl2.png")
	pulse1 = love.graphics.newImage("sprites/pulse1.png")
	pulse2 = love.graphics.newImage("sprites/pulse2.png")
	pulse1reverse = love.graphics.newImage("sprites/pulse1reverse.png")
	pulse2reverse = love.graphics.newImage("sprites/pulse2reverse.png")
	brainLight = love.graphics.newImage("sprites/brainLight.png")
	pulse1Up = true
	pulse2Up = true
	pulseScale = 1.0
	pulseScaleUp = true
	backgroundRotation = {angle = 0}
	backgroundRotationTweenLeft = tween.new(4, backgroundRotation, {angle = 0.5 * math.pi}, "inOutCubic")
	completedRight = false
	completedLeft = false
end

function interface.draw(scaleFactor)
    love.graphics.draw(backgroundPulse, 960, 540, backgroundRotation.angle, pulseScale * scaleFactor, pulseScale * scaleFactor, backgroundPulse:getWidth()*0.5, backgroundPulse:getHeight()*0.5)
	love.graphics.draw(brainGlow1, 247, 229, 0, pulseScale, pulseScale, brainGlow1:getWidth() * 0.5, brainGlow1:getHeight() * 0.5)
	love.graphics.draw(brainGlow2, 1674, 229, 0, pulseScale, pulseScale, brainGlow1:getWidth() * 0.5, brainGlow1:getHeight() * 0.5)
	love.graphics.draw(background, 0, 0)
	if pulse1Up then
		love.graphics.draw(pulse1, 100, 98)
	else
		love.graphics.draw(pulse1reverse, 100, 98)
	end
	if pulse2Up then
		love.graphics.draw(pulse2, 1571, 98)
	else
		love.graphics.draw(pulse2reverse, 1571, 98)
	end

	love.graphics.setBackgroundColor(65, 49, 66)
end

function interface.update(dt)
	if backgroundRotationTweenLeft then
		local completedLeft = backgroundRotationTweenLeft:update(dt)
		if completedLeft then
			backgroundRotationTweenLeft = nil
			backgroundRotationTweenRight = tween.new(4, backgroundRotation, {angle = -0.5 * math.pi}, "inOutCubic")
		end
	end
	if backgroundRotationTweenRight then
		local completedRight = backgroundRotationTweenRight:update(dt)
		if completedRight then
			backgroundRotationTweenRight = nil
			backgroundRotationTweenLeft = tween.new(4, backgroundRotation, {angle = 0.5 * math.pi}, "inOutCubic")
		end
	end
	if (pulseScale < 1.07) and pulseScaleUp then
		pulseScale = pulseScale * 60.27585 * dt
	elseif (pulseScaleUp == true) then
		pulseScaleUp = false
    elseif (pulseScale <= 1.0) then
    	pulseScaleUp = true
    else
    	pulseScale = pulseScale * 59.72415 * dt
    end
    --[[for i=10, 1, -1 do
	    if rectanglesLeft[i].xPos + rectanglesLeft[i].width < 960 then
	    	--rectanglesLeft[i].xPos = rectanglesLeft[i].xPos + rectanglesLeft[i].translationFactor*dt
	    	--rectanglesLeft[i].width = rectanglesLeft[i].width - rectanglesLeft[i].scaleFactor*dt

	    else
	    	--rectanglesLeft[i].xPos = 0
	    	--rectanglesLeft[i].width = rectanglesLeft[1].initWidth
	    en--]]

end

function interface.getDirectionLeftWave()
	return pulse1Up and 1 or -1
end

function interface.getDirectionRightWave()
	return pulse2Up and 1 or -1
end

function interface.keypressed(key)
	if key == "w" then
		pulse1Up = true
	elseif key == "s" then
		pulse1Up = false
	end 

	if key == "up" then
		pulse2Up = true
	elseif key == "down" then
		pulse2Up = false
	end

end

return interface