-- io.stdout:setvbuf("no") -- i want print() to show messages instantly

local currentScreen = require "gameScreen"
-- screen control
local intendedWidth = 1920
local intendedHeight = 1080
local widthResolution, heightResolution = 0, 0 
local scaleFactorWidth, scaleFactorHeight = 0, 0
local letterBoxOffset = 0
local fullscreen = true -- change to true in production

MS_PER_UPDATE = 1/60 -- 60 FPS
local lag = 0.0

function love.load()
	love.mouse.setVisible(false)
	love.window.setTitle("Brain Waves")
	love.window.setMode(960, 540, {vsync = false, fullscreen = fullscreen})
	widthResolution, heightResolution = love.graphics.getWidth(), love.graphics.getHeight()
	scaleFactorWidth = widthResolution / intendedWidth
	scaleFactorHeight = heightResolution / intendedHeight
	letterBoxOffset = math.max(0, (heightResolution - intendedHeight * scaleFactorWidth) / 2)
	love.keyboard.setKeyRepeat(false)
	currentScreen.load()
end


function love.draw(extrapolationFactor)
	love.graphics.push()
	love.graphics.translate(0, letterBoxOffset)
	love.graphics.scale(scaleFactorWidth, scaleFactorWidth)
	currentScreen.draw(extrapolationFactor)
	love.graphics.pop()
	love.graphics.setColor(0, 0, 0)
	love.graphics.rectangle('fill', 0, 0, love.graphics.getWidth(), letterBoxOffset)
	love.graphics.rectangle('fill', 0, love.graphics.getHeight() - letterBoxOffset, love.graphics.getWidth(), letterBoxOffset)
end

function love.update(dt)
	currentScreen.update(dt)
end

function love.run()
	if love.math then
		love.math.setRandomSeed(os.time())
	end
 
	if love.load then love.load(arg) end
 
	-- We don't want the first frame's dt to include time taken by love.load.
	if love.timer then love.timer.step() end
 
	local dt = 0
 
	-- Main loop time.
	while true do
		-- Process events.
		if love.event then
			love.event.pump()
			for name, a,b,c,d,e,f in love.event.poll() do
				if name == "quit" then
					if not love.quit or not love.quit() then
						return a
					end
				end
				love.handlers[name](a,b,c,d,e,f)
			end
		end
 
		-- Update dt, as we'll be passing it to update
		if love.timer then
			love.timer.step()
			dt = love.timer.getDelta()
		end

        lag = lag + dt
        while lag >= MS_PER_UPDATE do
            love.update(MS_PER_UPDATE)
            lag = lag - MS_PER_UPDATE
        end
 
		-- Call update and draw
		-- if love.update then love.update(MS_PER_UPDATE) end -- will pass 0 if love.timer is disabled
 
		if love.graphics and love.graphics.isActive() then
			love.graphics.clear(love.graphics.getBackgroundColor())
			love.graphics.origin()
			if love.draw then love.draw(lag/MS_PER_UPDATE) end
			love.graphics.present()
		end
 
		if love.timer then love.timer.sleep(0.001) end
	end
 
end

function love.keypressed(key)
	currentScreen.keypressed(key)
end

function love.keyreleased(key)
	currentScreen.keyreleased(key)
end

-- Callbacks
--[[function menuScreen.toCharacterSelectionScreen()
	menuScreen.unloadResources()
	currentScreen = characterSelectionScreen
	currentScreen.load(intendedWidth, intendedHeight)
end]]

--[[function gameScreen.toMenu()
	love.audio.stop()
	menuMusic:play()
	unloadNetworkResources()
	currentScreen = menuScreen
	currentScreen.load()
end]]